@@ -31,18 +31,18 @@
 
 using namespace Zeal::Core;
 
-Extractor::Extractor(QObject *parent) :
-    QObject(parent)
+Extractor::Extractor(QObject *parent)
+    : QObject(parent)
 {
 }
 
 void Extractor::extract(const QString &sourceFile, const QString &destination, const QString &root)
 {
     ExtractInfo info = {
-        archive_read_new(), // archiveHandle
-        sourceFile, // filePath
+        archive_read_new(),           // archiveHandle
+        sourceFile,                   // filePath
         QFileInfo(sourceFile).size(), // totalBytes
-        0 // extractedBytes
+        0                             // extractedBytes
     };
 
     archive_read_support_filter_all(info.archiveHandle);
@@ -56,9 +56,12 @@ void Extractor::extract(const QString &sourceFile, const QString &destination, c
 
     QDir destinationDir(destination);
     if (!root.isEmpty()) {
-        destinationDir = destinationDir.filePath(root);
+        destinationDir.setPath(destinationDir.filePath(root));
     }
 
+    // Destination directory must be created before any other files.
+    destinationDir.mkpath(QLatin1String("."));
+
     // TODO: Do not strip root directory in archive if it equals to 'root'
     archive_entry *entry;
     while (archive_read_next_header(info.archiveHandle, &entry) == ARCHIVE_OK) {
@@ -79,7 +82,9 @@ void Extractor::extract(const QString &sourceFile, const QString &destination, c
         if (filetype == S_IFDIR) {
             QDir().mkpath(QFileInfo(filePath).absolutePath());
             continue;
-        } else if (filetype != S_IFREG) {
+        }
+
+        if (filetype != S_IFREG) {
             qWarning("Unsupported filetype %d for %s!", filetype, qPrintable(pathname));
             continue;
         }
