@@ -35,36 +35,39 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QRegularExpression>
-#include <QVariant>
 #include <QVarLengthArray>
+#include <QVariant>
 
 #include <sqlite3.h>
 
 #include <cstring>
+#include <utility>
 
 using namespace Zeal::Registry;
 
 namespace {
-const char IndexNamePrefix[] = "__zi_name"; // zi - Zeal index
-const char IndexNameVersion[] = "0001"; // Current index version
+constexpr char IndexNamePrefix[] = "__zi_name"; // zi - Zeal index
+constexpr char IndexNameVersion[] = "0001";     // Current index version
+
+constexpr char NotFoundPageUrl[] = "qrc:///browser/404.html";
 
 namespace InfoPlist {
-const char CFBundleName[] = "CFBundleName";
+constexpr char CFBundleName[] = "CFBundleName";
 //const char CFBundleIdentifier[] = "CFBundleIdentifier";
-const char DashDocSetFamily[] = "DashDocSetFamily";
-const char DashDocSetKeyword[] = "DashDocSetKeyword";
-const char DashDocSetPluginKeyword[] = "DashDocSetPluginKeyword";
-const char DashIndexFilePath[] = "dashIndexFilePath";
-const char DocSetPlatformFamily[] = "DocSetPlatformFamily";
+constexpr char DashDocSetFamily[] = "DashDocSetFamily";
+constexpr char DashDocSetKeyword[] = "DashDocSetKeyword";
+constexpr char DashDocSetPluginKeyword[] = "DashDocSetPluginKeyword";
+constexpr char DashIndexFilePath[] = "dashIndexFilePath";
+constexpr char DocSetPlatformFamily[] = "DocSetPlatformFamily";
 //const char IsDashDocset[] = "isDashDocset";
-//const char IsJavaScriptEnabled[] = "isJavaScriptEnabled";
-}
-}
+constexpr char IsJavaScriptEnabled[] = "isJavaScriptEnabled";
+} // namespace InfoPlist
+} // namespace
 
 static void sqliteScoreFunction(sqlite3_context *context, int argc, sqlite3_value **argv);
 
-Docset::Docset(const QString &path) :
-    m_path(path)
+Docset::Docset(QString path)
+    : m_path(std::move(path))
 {
     QDir dir(m_path);
     if (!dir.exists())
@@ -73,7 +76,8 @@ Docset::Docset(const QString &path) :
     loadMetadata();
 
     // Attempt to find the icon in any supported format
-    for (const QString &iconFile : dir.entryList({QStringLiteral("icon.*")}, QDir::Files)) {
+    const auto iconFiles = dir.entryList({QStringLiteral("icon.*")}, QDir::Files);
+    for (const QString &iconFile : iconFiles) {
         m_icon = QIcon(dir.filePath(iconFile));
         if (!m_icon.availableSizes().isEmpty())
             break;
@@ -163,16 +167,39 @@ Docset::Docset(const QString &path) :
         }
     }
 
+    if (plist.contains(InfoPlist::IsJavaScriptEnabled)) {
+        m_isJavaScriptEnabled = plist[InfoPlist::IsJavaScriptEnabled].toBool();
+    }
+
     m_keywords.removeDuplicates();
 
-    // Prefer index path provided by the docset over metadata.
+    // Determine index page. This is ridiculous.
+    const QString mdIndexFilePath = m_indexFilePath; // Save path from the metadata.
+
+    // Prefer index path provided by the docset.
     if (plist.contains(InfoPlist::DashIndexFilePath)) {
-        m_indexFileUrl = createPageUrl(plist[InfoPlist::DashIndexFilePath].toString());
-    } else if (m_indexFileUrl.isEmpty()) {
-        if (dir.exists(QStringLiteral("index.html")))
-            m_indexFileUrl = createPageUrl(QStringLiteral("index.html"));
-        else
-            qWarning("Cannot determine index file for docset %s", qPrintable(m_name));
+        const QString indexFilePath = plist[InfoPlist::DashIndexFilePath].toString();
+        if (dir.exists(indexFilePath)) {
+            m_indexFilePath = indexFilePath;
+        }
+    }
+
+    // Check the metadata.
+    if (m_indexFilePath.isEmpty() && !mdIndexFilePath.isEmpty() && dir.exists(mdIndexFilePath)) {
+        m_indexFilePath = mdIndexFilePath;
+    }
+
+    // What if there is index.html.
+    if (m_indexFilePath.isEmpty() && dir.exists(QStringLiteral("index.html"))) {
+        m_indexFilePath = QStringLiteral("index.html");
+    }
+
+    // Log if unable to determine the index page. Otherwise the path will be set in setBaseUrl().
+    if (m_indexFilePath.isEmpty()) {
+        qWarning("Cannot determine index file for docset %s", qPrintable(m_name));
+        m_indexFileUrl.setUrl(NotFoundPageUrl);
+    } else {
+        m_indexFileUrl = createPageUrl(m_indexFilePath);
     }
 
     countSymbols();
@@ -208,7 +235,7 @@ QString Docset::version() const
     return m_version;
 }
 
-QString Docset::revision() const
+int Docset::revision() const
 {
     return m_revision;
 }
@@ -256,7 +283,7 @@ int Docset::symbolCount(const QString &symbolType) const
     return m_symbolCounts.value(symbolType);
 }
 
-const QMap<QString, QUrl> &Docset::symbols(const QString &symbolType) const
+const QMultiMap<QString, QUrl> &Docset::symbols(const QString &symbolType) const
 {
     if (!m_symbols.contains(symbolType))
         loadSymbols(symbolType);
@@ -267,7 +294,7 @@ QList<SearchResult> Docset::search(const QString &query, const CancellationToken
 {
     QString sql;
     if (m_type == Docset::Type::Dash) {
-        if (m_fuzzySearchEnabled) {
+        if (m_isFuzzySearchEnabled) {
             sql = QStringLiteral("SELECT name, type, path, '', zealScore('%1', name) as score"
                                  "  FROM searchIndex"
                                  "  WHERE score > 0");
@@ -277,7 +304,7 @@ QList<SearchResult> Docset::search(const QString &query, const CancellationToken
                                  "  WHERE (name LIKE '%%1%' ESCAPE '\\')");
         }
     } else {
-        if (m_fuzzySearchEnabled) {
+        if (m_isFuzzySearchEnabled) {
             sql = QStringLiteral("SELECT name, type, path, fragment, zealScore('%1', name) as score"
                                  "  FROM searchIndex"
                                  "  WHERE score > 0");
@@ -312,17 +339,12 @@ QList<SearchResult> Docset::search(const QString &query, const CancellationToken
 
 QList<SearchResult> Docset::relatedLinks(const QUrl &url) const
 {
-    QList<SearchResult> results;
+    if (!m_baseUrl.isParentOf(url)) {
+        return {};
+    }
 
-    // Strip docset path and anchor from url
-    const QString dir = documentPath();
-    QString urlPath = url.path();
-    int dirPosition = urlPath.indexOf(dir);
-    QString path = url.path().mid(dirPosition + dir.size() + 1);
-
-    // Get the url without the #anchor.
-    QUrl cleanUrl(path);
-    cleanUrl.setFragment(QString());
+    // Get page path within the docset.
+    const QString path = url.path().mid(m_baseUrl.path().length() + 1);
 
     // Prepare the query to look up all pages with the same url.
     QString sql;
@@ -336,7 +358,9 @@ QList<SearchResult> Docset::relatedLinks(const QUrl &url) const
                              "  WHERE path = \"%1\" AND fragment IS NOT NULL");
     }
 
-    m_db->prepare(sql.arg(cleanUrl.toString()));
+    QList<SearchResult> results;
+
+    m_db->prepare(sql.arg(path));
     while (m_db->next()) {
         results.append({m_db->value(0).toString(),
                         parseSymbolType(m_db->value(1).toString()),
@@ -344,8 +368,9 @@ QList<SearchResult> Docset::relatedLinks(const QUrl &url) const
                         const_cast<Docset *>(this), 0});
     }
 
-    if (results.size() == 1)
-        results.clear();
+    if (results.size() == 1) {
+        return {};
+    }
 
     return results;
 }
@@ -376,7 +401,7 @@ void Docset::loadMetadata()
     m_name = jsonObject[QStringLiteral("name")].toString();
     m_title = jsonObject[QStringLiteral("title")].toString();
     m_version = jsonObject[QStringLiteral("version")].toString();
-    m_revision = jsonObject[QStringLiteral("revision")].toString();
+    m_revision = jsonObject[QStringLiteral("revision")].toString().toInt();
 
     if (jsonObject.contains(QStringLiteral("feed_url"))) {
         m_feedUrl = jsonObject[QStringLiteral("feed_url")].toString();
@@ -386,12 +411,17 @@ void Docset::loadMetadata()
         const QJsonObject extra = jsonObject[QStringLiteral("extra")].toObject();
 
         if (extra.contains(QStringLiteral("indexFilePath"))) {
-            m_indexFileUrl = createPageUrl(extra[QStringLiteral("indexFilePath")].toString());
+            m_indexFilePath = extra[QStringLiteral("indexFilePath")].toString();
         }
 
         if (extra.contains(QStringLiteral("keywords"))) {
-            for (const QJsonValueRef kw : extra[QStringLiteral("keywords")].toArray())
+            for (const QJsonValueRef kw : extra[QStringLiteral("keywords")].toArray()) {
                 m_keywords << kw.toString();
+            }
+        }
+
+        if (extra.contains(QStringLiteral("isJavaScriptEnabled"))) {
+            m_isJavaScriptEnabled = extra[QStringLiteral("isJavaScriptEnabled")].toBool();
         }
     }
 }
@@ -416,7 +446,7 @@ void Docset::countSymbols()
         }
 
         const QString symbolType = parseSymbolType(symbolTypeStr);
-        m_symbolStrings.insertMulti(symbolType, symbolTypeStr);
+        m_symbolStrings.insert(symbolType, symbolTypeStr);
         m_symbolCounts[symbolType] += m_db->value(1).toInt();
     }
 }
@@ -424,8 +454,13 @@ void Docset::countSymbols()
 // TODO: Fetch and cache only portions of symbols
 void Docset::loadSymbols(const QString &symbolType) const
 {
-    for (const QString &symbol : m_symbolStrings.values(symbolType))
-        loadSymbols(symbolType, symbol);
+    // Iterator `it` is a QPair<QMap::const_iterator, QMap::const_iterator>,
+    // with it.first and it.second respectively pointing to the start and the end
+    // of the range of nodes having symbolType as key. It effectively represents a
+    // contiguous view over the nodes with a specified key.
+    for (auto it = qAsConst(m_symbolStrings).equal_range(symbolType); it.first != it.second; ++it.first) {
+        loadSymbols(symbolType, it.first.value());
+    }
 }
 
 void Docset::loadSymbols(const QString &symbolType, const QString &symbolString) const
@@ -448,10 +483,12 @@ void Docset::loadSymbols(const QString &symbolType, const QString &symbolString)
         return;
     }
 
-    QMap<QString, QUrl> &symbols = m_symbols[symbolType];
-    while (m_db->next())
-        symbols.insertMulti(m_db->value(0).toString(),
-                            createPageUrl(m_db->value(1).toString(), m_db->value(2).toString()));
+    QMultiMap<QString, QUrl> &symbols = m_symbols[symbolType];
+    while (m_db->next()) {
+        symbols.insert(m_db->value(0).toString(),
+                       createPageUrl(m_db->value(1).toString(),
+                                     m_db->value(2).toString()));
+    }
 }
 
 void Docset::createIndex()
@@ -482,10 +519,11 @@ void Docset::createIndex()
     }
 
     // Drop old indexes
-    for (const QString &oldIndexName : oldIndexes)
+    for (const QString &oldIndexName : qAsConst(oldIndexes)) {
         m_db->execute(indexDropQuery.arg(oldIndexName));
+    }
 
-    m_db->execute(indexCreateQuery.arg(IndexNamePrefix, IndexNameVersion, tableName, columnName));
+    m_db->execute(indexCreateQuery.arg(IndexNamePrefix).arg(IndexNameVersion).arg(tableName).arg(columnName));
 }
 
 void Docset::createView()
@@ -523,12 +561,13 @@ QUrl Docset::createPageUrl(const QString &path, const QString &fragment) const
         realFragment = fragment;
     }
 
-    static const QRegularExpression dashEntryRegExp(QLatin1String("<dash_entry_.*>"));
+    static const QRegularExpression dashEntryRegExp(QStringLiteral("<dash_entry_.*>"));
     realPath.remove(dashEntryRegExp);
     realFragment.remove(dashEntryRegExp);
 
-    // Absolute file path is required here to handle relative path to the docset storage (see #806).
-    QUrl url = QUrl::fromLocalFile(QDir(documentPath()).absoluteFilePath(realPath));
+    QUrl url = m_baseUrl;
+    url.setPath(m_baseUrl.path() + "/" + realPath, QUrl::TolerantMode);
+
     if (!realFragment.isEmpty()) {
         if (realFragment.startsWith(QLatin1String("//apple_ref"))
                 || realFragment.startsWith(QLatin1String("//dash_ref"))) {
@@ -653,7 +692,10 @@ QString Docset::parseSymbolType(const QString &str)
         // Protocol
         {QStringLiteral("intf"), QStringLiteral("Protocol")},
         // Structure
+        {QStringLiteral("_Struct"), QStringLiteral("Structure")},
+        {QStringLiteral("_Structs"), QStringLiteral("Structure")},
         {QStringLiteral("struct"), QStringLiteral("Structure")},
+        {QStringLiteral("Сontrol Structure"), QStringLiteral("Structure")},
         {QStringLiteral("Data Structures"), QStringLiteral("Structure")},
         {QStringLiteral("Struct"), QStringLiteral("Structure")},
         // Type
@@ -672,14 +714,33 @@ QString Docset::parseSymbolType(const QString &str)
     return aliases.value(str, str);
 }
 
+QUrl Docset::baseUrl() const
+{
+    return m_baseUrl;
+}
+
+void Docset::setBaseUrl(const QUrl &baseUrl)
+{
+    m_baseUrl = baseUrl;
+
+    if (!m_indexFilePath.isEmpty()) {
+        m_indexFileUrl = createPageUrl(m_indexFilePath);
+    }
+}
+
 bool Docset::isFuzzySearchEnabled() const
 {
-    return m_fuzzySearchEnabled;
+    return m_isFuzzySearchEnabled;
 }
 
 void Docset::setFuzzySearchEnabled(bool enabled)
 {
-    m_fuzzySearchEnabled = enabled;
+    m_isFuzzySearchEnabled = enabled;
+}
+
+bool Docset::isJavaScriptEnabled() const
+{
+    return m_isJavaScriptEnabled;
 }
 
 /**
@@ -691,16 +752,18 @@ void Docset::setFuzzySearchEnabled(bool enabled)
  */
 static int scoreFuzzy(const char *str, int index, int length)
 {
+    // Score between 66..99, if the match follows a dot, or starts the string.
     if (index == 0 || str[index - 1] == '.') {
-        // score between 66..99, if the match follows a dot, or starts the string
         return qMax(66, 100 - length);
-    } else if (str[index + length] == 0) {
-        // score between 33..66, if the match is at the end of the string
-        return qMax(33, 67 - length);
-    } else {
-        // score between 1..33 otherwise (match in the middle of the string)
-        return qMax(1, 34 - length);
     }
+
+    // Score between 33..66, if the match is at the end of the string.
+    if (str[index + length] == 0) {
+        return qMax(33, 67 - length);
+    }
+
+    // Score between 1..33 otherwise (match in the middle of the string).
+    return qMax(1, 34 - length);
 }
 
 // Based on https://github.com/bevacqua/fuzzysearch
@@ -726,7 +789,7 @@ static void matchFuzzy(const char *needle, int needleLength,
         while (j < haystackLength) {
             if (needle[i] == haystack[j++]) {
                 if (*start == -1) {
-                    *start = j;  // first matched char
+                    *start = j; // first matched char
 
                     // try starting the search later in case the first character occurs again later
                     int recursiveStart;
@@ -816,11 +879,12 @@ static int scoreExact(int matchIndex, int matchLen, const char *value, int value
             // (2) Remove one point for each unmatched character
             //     following the query.
             int i = matchIndex - 2;
-            while (i >= 0 && value[i] != DOT)
+            while (i >= 0 && value[i] != DOT) {
                 --i;
+            }
 
-            score -= (matchIndex - i)                      // (1)
-                    + (valueLen - matchLen - matchIndex);  // (2)
+            score -= (matchIndex - i)                     // (1)
+                    + (valueLen - matchLen - matchIndex); // (2)
         }
 
         // Remove one point for each dot preceding the query, except for the
@@ -853,7 +917,7 @@ static inline int scoreFunction(const char *needleOrig, const char *haystackOrig
         if ((i > 0 && needleOrig[i - 1] == ':' && c == ':') // C++ (::)
                 || c == '/' || c == '_' || c == ' ') { // Go, some Guides
             needle[j] = '.';
-        } else if (c >= 'A' && c <= 'Z')  {
+        } else if (c >= 'A' && c <= 'Z') {
             needle[j] = c + 32;
         } else {
             needle[j] = c;
@@ -865,7 +929,7 @@ static inline int scoreFunction(const char *needleOrig, const char *haystackOrig
         if ((i > 0 && haystackOrig[i - 1] == ':' && c == ':') // C++ (::)
                 || c == '/' || c == '_' || c == ' ') { // Go, some Guides
             haystack[j] = '.';
-        } else if (c >= 'A' && c <= 'Z')  {
+        } else if (c >= 'A' && c <= 'Z') {
             haystack[j] = c + 32;
         } else {
             haystack[j] = c;
@@ -888,10 +952,12 @@ static inline int scoreFunction(const char *needleOrig, const char *haystackOrig
                    &matchIndex, &matchLength);
     }
 
-    if (matchIndex == -1 && exactIndex == -1) { // no match
-        // simply return 0
+    if (matchIndex == -1 && exactIndex == -1) {
+        // no match
         return 0;
-    } else if (exactIndex != -1) {
+    }
+
+    if (exactIndex != -1) {
         // +100 to make sure exact matches are always on top.
         score = scoreExact(exactIndex, needleLength, haystack.data(), haystackLength) + 100;
     } else {
@@ -921,10 +987,10 @@ static inline int scoreFunction(const char *needleOrig, const char *haystackOrig
 
 static void sqliteScoreFunction(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
-    Q_UNUSED(argc);
+    Q_UNUSED(argc)
 
-    const char *needle = reinterpret_cast<const char *>(sqlite3_value_text(argv[0]));
-    const char *haystack = reinterpret_cast<const char *>(sqlite3_value_text(argv[1]));
+    auto needle = reinterpret_cast<const char *>(sqlite3_value_text(argv[0]));
+    auto haystack = reinterpret_cast<const char *>(sqlite3_value_text(argv[1]));
 
     sqlite3_result_int(context, scoreFunction(needle, haystack));
 }
