@@ -21,11 +21,12 @@
 **
 ****************************************************************************/
 
-#ifndef LISTMODEL_H
-#define LISTMODEL_H
+#ifndef ZEAL_REGISTRY_LISTMODEL_H
+#define ZEAL_REGISTRY_LISTMODEL_H
+
+#include <util/caseinsensitivemap.h>
 
 #include <QAbstractItemModel>
-#include <QMap>
 
 namespace Zeal {
 namespace Registry {
@@ -33,11 +34,11 @@ namespace Registry {
 class Docset;
 class DocsetRegistry;
 
-class ListModel : public QAbstractItemModel
+class ListModel final : public QAbstractItemModel
 {
     Q_OBJECT
+    Q_DISABLE_COPY(ListModel)
 public:
-    explicit ListModel(DocsetRegistry *docsetRegistry, QObject *parent = nullptr);
     ~ListModel() override;
 
     QVariant data(const QModelIndex &index, int role) const override;
@@ -51,6 +52,8 @@ private slots:
     void removeDocset(const QString &name);
 
 private:
+    friend class DocsetRegistry;
+
     enum Level {
         RootLevel,
         DocsetLevel,
@@ -58,6 +61,8 @@ private:
         SymbolLevel
     };
 
+    explicit ListModel(DocsetRegistry *docsetRegistry);
+
     inline static QString pluralize(const QString &s);
     inline static Level indexLevel(const QModelIndex &index);
 
@@ -76,10 +81,12 @@ private:
         QList<GroupItem *> groups;
     };
 
-    QMap<QString, DocsetItem *> m_docsetItems;
+    inline DocsetItem *itemInRow(int row) const;
+
+    Util::CaseInsensitiveMap<DocsetItem *> m_docsetItems;
 };
 
 } // namespace Registry
 } // namespace Zeal
 
-#endif // LISTMODEL_H
+#endif // ZEAL_REGISTRY_LISTMODEL_H
