@@ -25,7 +25,6 @@
 #include "ui_docsetsdialog.h"
 
 #include "docsetlistitemdelegate.h"
-#include "progressitemdelegate.h"
 
 #include <core/application.h>
 #include <core/filemanager.h>
@@ -33,7 +32,6 @@
 #include <registry/docset.h>
 #include <registry/docsetregistry.h>
 #include <registry/itemdatarole.h>
-#include <registry/listmodel.h>
 
 #include <QClipboard>
 #include <QDateTime>
@@ -41,12 +39,15 @@
 #include <QInputDialog>
 #include <QJsonArray>
 #include <QJsonDocument>
+#include <QLocale>
 #include <QMessageBox>
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QTemporaryFile>
 #include <QUrl>
 
+#include <memory>
+
 using namespace Zeal;
 using namespace Zeal::WidgetUi;
 
@@ -55,50 +56,44 @@ extern Q_CORE_EXPORT int qt_ntfs_permission_lookup;
 #endif
 
 namespace {
-const char ApiServerUrl[] = "http://api.zealdocs.org/v1";
-const char RedirectServerUrl[] = "https://go.zealdocs.org";
+constexpr char ApiServerUrl[] = "https://api.zealdocs.org/v1";
+constexpr char RedirectServerUrl[] = "https://go.zealdocs.org/d/%1/%2/latest";
 // TODO: Each source plugin should have its own cache
-const char DocsetListCacheFileName[] = "com.kapeli.json";
+constexpr char DocsetListCacheFileName[] = "com.kapeli.json";
 
 // TODO: Make the timeout period configurable
 constexpr int CacheTimeout = 24 * 60 * 60 * 1000; // 24 hours in microseconds
 
 // QNetworkReply properties
-const char DocsetNameProperty[] = "docsetName";
-const char DownloadTypeProperty[] = "downloadType";
-const char DownloadPreviousReceived[] = "downloadPreviousReceived";
-const char ListItemIndexProperty[] = "listItem";
-}
+constexpr char DocsetNameProperty[] = "docsetName";
+constexpr char DownloadTypeProperty[] = "downloadType";
+constexpr char ListItemIndexProperty[] = "listItem";
+} // namespace
 
-DocsetsDialog::DocsetsDialog(Core::Application *app, QWidget *parent) :
-    QDialog(parent),
-    ui(new Ui::DocsetsDialog()),
-    m_application(app),
-    m_docsetRegistry(app->docsetRegistry())
+DocsetsDialog::DocsetsDialog(Core::Application *app, QWidget *parent)
+    : QDialog(parent)
+    , ui(new Ui::DocsetsDialog())
+    , m_application(app)
+    , m_docsetRegistry(app->docsetRegistry())
 {
     ui->setupUi(this);
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     loadDocsetList();
 
-#ifdef Q_OS_WIN32
-    qt_ntfs_permission_lookup++;
-#endif
+    m_isStorageReadOnly = !isDirWritable(m_application->settings()->docsetPath);
 
-    m_isStorageReadOnly = !QFileInfo(m_application->settings()->docsetPath).isWritable();
-
-#ifdef Q_OS_WIN32
-    qt_ntfs_permission_lookup--;
-#endif
-
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     ui->availableDocsetList->setAttribute(Qt::WA_MacShowFocusRect, false);
     ui->installedDocsetList->setAttribute(Qt::WA_MacShowFocusRect, false);
 #endif
 
-    ui->combinedProgressBar->hide();
-    ui->cancelButton->hide();
-    ui->readOnlyLabel->setVisible(m_isStorageReadOnly);
+    ui->statusLabel->clear(); // Clear text shown in the designer mode.
+    ui->storageStatusLabel->setVisible(m_isStorageReadOnly);
+
+    const QFileInfo fi(m_application->settings()->docsetPath);
+    ui->storageStatusLabel->setText(fi.exists() ? tr("<b>Docset storage is read only.</b>")
+                                                : tr("<b>Docset storage does not exist.</b>"));
 
     connect(m_application, &Core::Application::extractionCompleted,
             this, &DocsetsDialog::extractionCompleted);
@@ -107,16 +102,24 @@ DocsetsDialog::DocsetsDialog(Core::Application *app, QWidget *parent) :
     connect(m_application, &Core::Application::extractionProgress,
             this, &DocsetsDialog::extractionProgress);
 
-    connect(ui->cancelButton, &QPushButton::clicked, this, &DocsetsDialog::cancelDownloads);
+    // Setup signals & slots
+    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, [this](QAbstractButton *button) {
+        if (button == ui->buttonBox->button(QDialogButtonBox::Cancel)) {
+            cancelDownloads();
+            return;
+        }
+
+        if (button == ui->buttonBox->button(QDialogButtonBox::Close)) {
+            close();
+            return;
+        }
+    });
 
     setupInstalledDocsetsTab();
     setupAvailableDocsetsTab();
 
     if (m_isStorageReadOnly) {
         disableControls();
-
-        // Updating the docset list is fine;
-        ui->refreshButton->setEnabled(true);
     }
 }
 
@@ -125,17 +128,6 @@ DocsetsDialog::~DocsetsDialog()
     delete ui;
 }
 
-void DocsetsDialog::reject()
-{
-    if (m_replies.isEmpty() && m_tmpFiles.isEmpty()) {
-        QDialog::reject();
-        return;
-    }
-
-    QMessageBox::information(this, QStringLiteral("Zeal"),
-                             tr("Please wait for all operations to finish."));
-}
-
 void DocsetsDialog::addDashFeed()
 {
     QString clipboardText = QApplication::clipboard()->text();
@@ -144,6 +136,7 @@ void DocsetsDialog::addDashFeed()
 
     QString feedUrl = QInputDialog::getText(this, QStringLiteral("Zeal"), tr("Feed URL:"),
                                             QLineEdit::Normal, clipboardText);
+    feedUrl = feedUrl.trimmed();
     if (feedUrl.isEmpty())
         return;
 
@@ -158,7 +151,8 @@ void DocsetsDialog::addDashFeed()
 
 void DocsetsDialog::updateSelectedDocsets()
 {
-    for (const QModelIndex &index : ui->installedDocsetList->selectionModel()->selectedRows()) {
+    const auto selectedRows = ui->installedDocsetList->selectionModel()->selectedRows();
+    for (const QModelIndex &index : selectedRows) {
         if (!index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool())
             continue;
 
@@ -180,13 +174,13 @@ void DocsetsDialog::updateAllDocsets()
 
 void DocsetsDialog::removeSelectedDocsets()
 {
-    QItemSelectionModel *selectonModel = ui->installedDocsetList->selectionModel();
-    if (!selectonModel->hasSelection())
+    QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
+    if (!selectionModel->hasSelection())
         return;
 
     int ret;
 
-    const QModelIndexList selectedIndexes = selectonModel->selectedRows();
+    const QModelIndexList selectedIndexes = selectionModel->selectedRows();
     if (selectedIndexes.size() == 1) {
         const QString docsetTitle = selectedIndexes.first().data().toString();
         ret = QMessageBox::question(this, QStringLiteral("Zeal"),
@@ -197,16 +191,19 @@ void DocsetsDialog::removeSelectedDocsets()
                                        selectedIndexes.size()));
     }
 
-    if (ret == QMessageBox::No)
+    if (ret == QMessageBox::No) {
         return;
+    }
 
     // Gather names first, because model indicies become invalid when docsets are removed.
     QStringList names;
-    for (const QModelIndex &index : selectedIndexes)
+    for (const QModelIndex &index : selectedIndexes) {
         names.append(index.data(Registry::ItemDataRole::DocsetNameRole).toString());
+    }
 
-    for (const QString &name : names)
+    for (const QString &name : names) {
         removeDocset(name);
+    }
 }
 
 void DocsetsDialog::updateDocsetFilter(const QString &filterString)
@@ -227,17 +224,18 @@ void DocsetsDialog::updateDocsetFilter(const QString &filterString)
 void DocsetsDialog::downloadSelectedDocsets()
 {
     QItemSelectionModel *selectionModel = ui->availableDocsetList->selectionModel();
-    for (const QModelIndex &index : selectionModel->selectedRows()) {
+    const auto selectedRows = selectionModel->selectedRows();
+    for (const QModelIndex &index : selectedRows) {
         selectionModel->select(index, QItemSelectionModel::Deselect);
 
         // Do nothing if a download is already in progress.
-        if (index.data(ProgressItemDelegate::ShowProgressRole).toBool())
+        if (index.data(DocsetListItemDelegate::ShowProgressRole).toBool())
             continue;
 
         QAbstractItemModel *model = ui->availableDocsetList->model();
-        model->setData(index, tr("Downloading: %p%"), ProgressItemDelegate::FormatRole);
-        model->setData(index, 0, ProgressItemDelegate::ValueRole);
-        model->setData(index, true, ProgressItemDelegate::ShowProgressRole);
+        model->setData(index, tr("Downloading: %p%"), DocsetListItemDelegate::FormatRole);
+        model->setData(index, 0, DocsetListItemDelegate::ValueRole);
+        model->setData(index, true, DocsetListItemDelegate::ShowProgressRole);
 
         downloadDashDocset(index);
     }
@@ -257,10 +255,10 @@ void DocsetsDialog::downloadCompleted()
 
     if (reply->error() != QNetworkReply::NoError) {
         if (reply->error() != QNetworkReply::OperationCanceledError) {
-            const int ret = QMessageBox::warning(this, QStringLiteral("Zeal"), reply->errorString(),
-                                                 QMessageBox::Retry | QMessageBox::Default,
-                                                 QMessageBox::Cancel | QMessageBox::Escape,
-                                                 QMessageBox::NoButton);
+            const QString msg = tr("Download failed!<br><br><b>Error:</b> %1<br><b>URL:</b> %2")
+                    .arg(reply->errorString(), reply->request().url().toString());
+            const int ret = QMessageBox::warning(this, QStringLiteral("Zeal"), msg,
+                                                 QMessageBox::Retry | QMessageBox::Cancel);
 
             if (ret == QMessageBox::Retry) {
                 QNetworkReply *newReply = download(reply->request().url());
@@ -277,29 +275,10 @@ void DocsetsDialog::downloadCompleted()
             QListWidgetItem *listItem = ui->availableDocsetList->item(
                         reply->property(ListItemIndexProperty).toInt(&ok));
             if (ok && listItem)
-                listItem->setData(ProgressItemDelegate::ShowProgressRole, false);
+                listItem->setData(DocsetListItemDelegate::ShowProgressRole, false);
         }
 
-        resetProgress();
-        return;
-    }
-
-    QUrl redirectUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
-    if (redirectUrl.isValid()) {
-        if (redirectUrl.isRelative())
-            redirectUrl = reply->request().url().resolved(redirectUrl);
-
-        // TODO: Verify if scheme can be missing
-        if (redirectUrl.scheme().isEmpty())
-            redirectUrl.setScheme(reply->request().url().scheme());
-
-        QNetworkReply *newReply = download(redirectUrl);
-
-        // Copy properties
-        newReply->setProperty(DocsetNameProperty, reply->property(DocsetNameProperty));
-        newReply->setProperty(DownloadTypeProperty, reply->property(DownloadTypeProperty));
-        newReply->setProperty(ListItemIndexProperty, reply->property(ListItemIndexProperty));
-
+        updateStatus();
         return;
     }
 
@@ -311,8 +290,9 @@ void DocsetsDialog::downloadCompleted()
         if (file->open(QIODevice::WriteOnly))
             file->write(data);
 
-        ui->lastUpdatedLabel->setText(QFileInfo(file->fileName())
-                                      .lastModified().toString(Qt::SystemLocaleShortDate));
+        const QString updateTime = QFileInfo(file->fileName())
+                .lastModified().toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat));
+        ui->lastUpdatedLabel->setText(updateTime);
 
         QJsonParseError jsonError;
         const QJsonDocument jsonDoc = QJsonDocument::fromJson(data, &jsonError);
@@ -379,19 +359,21 @@ void DocsetsDialog::downloadCompleted()
 
         QTemporaryFile *tmpFile = m_tmpFiles[docsetName];
         if (!tmpFile) {
-            tmpFile = new QTemporaryFile(this);
+            tmpFile = new QTemporaryFile(QStringLiteral("%1/%2.XXXXXX.tmp").arg(Core::Application::cacheLocation(), docsetName), this);
             tmpFile->open();
             m_tmpFiles.insert(docsetName, tmpFile);
         }
 
-        while (reply->bytesAvailable())
-            tmpFile->write(reply->read(1024 * 1024)); // Use small chunks
+        while (reply->bytesAvailable()) {
+            tmpFile->write(reply->read(1024 * 1024)); // Use small chunks.
+        }
+
         tmpFile->close();
 
         QListWidgetItem *item = findDocsetListItem(docsetName);
         if (item) {
-            item->setData(ProgressItemDelegate::ValueRole, 0);
-            item->setData(ProgressItemDelegate::FormatRole, tr("Installing: %p%"));
+            item->setData(DocsetListItemDelegate::ValueRole, 0);
+            item->setData(DocsetListItemDelegate::FormatRole, tr("Installing: %p%"));
         }
 
         m_application->extract(tmpFile->fileName(), m_application->settings()->docsetPath,
@@ -400,9 +382,8 @@ void DocsetsDialog::downloadCompleted()
     }
     }
 
-    // If all enqueued downloads have finished executing
-    if (m_replies.isEmpty())
-        resetProgress();
+    // If all enqueued downloads have finished executing.
+    updateStatus();
 }
 
 // creates a total download progress for multiple QNetworkReplies
@@ -412,7 +393,7 @@ void DocsetsDialog::downloadProgress(qint64 received, qint64 total)
     if (total == -1 || received < 10240)
         return;
 
-    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
+    auto reply = qobject_cast<QNetworkReply *>(sender());
     if (!reply || !reply->isOpen())
         return;
 
@@ -421,7 +402,7 @@ void DocsetsDialog::downloadProgress(qint64 received, qint64 total)
 
         QTemporaryFile *tmpFile = m_tmpFiles[docsetName];
         if (!tmpFile) {
-            tmpFile = new QTemporaryFile(this);
+            tmpFile = new QTemporaryFile(QStringLiteral("%1/%2.XXXXXX.tmp").arg(Core::Application::cacheLocation(), docsetName), this);
             tmpFile->open();
             m_tmpFiles.insert(docsetName, tmpFile);
         }
@@ -432,20 +413,9 @@ void DocsetsDialog::downloadProgress(qint64 received, qint64 total)
     // Try to get the item associated to the request
     QListWidgetItem *item
             = ui->availableDocsetList->item(reply->property(ListItemIndexProperty).toInt());
-    if (item)
-        item->setData(ProgressItemDelegate::ValueRole, percent(received, total));
-
-    qint64 previousReceived = 0;
-    const QVariant previousReceivedVariant = reply->property(DownloadPreviousReceived);
-    if (!previousReceivedVariant.isValid())
-        m_combinedTotal += total;
-    else
-        previousReceived = previousReceivedVariant.toLongLong();
-
-    m_combinedReceived += received - previousReceived;
-    reply->setProperty(DownloadPreviousReceived, received);
-
-    updateCombinedProgress();
+    if (item) {
+        item->setData(DocsetListItemDelegate::ValueRole, percent(received, total));
+    }
 }
 
 void DocsetsDialog::extractionCompleted(const QString &filePath)
@@ -456,7 +426,7 @@ void DocsetsDialog::extractionCompleted(const QString &filePath)
     const QString docsetPath = dataDir.filePath(docsetName + QLatin1String(".docset"));
 
     // Write metadata about docset
-    Registry::DocsetMetadata metadata = m_availableDocsets.contains(docsetName)
+    Registry::DocsetMetadata metadata = m_availableDocsets.count(docsetName)
             ? m_availableDocsets[docsetName] : m_userFeeds[docsetName];
     metadata.save(docsetPath, metadata.latestVersion());
 
@@ -465,11 +435,12 @@ void DocsetsDialog::extractionCompleted(const QString &filePath)
     QListWidgetItem *listItem = findDocsetListItem(docsetName);
     if (listItem) {
         listItem->setHidden(true);
-        listItem->setData(ProgressItemDelegate::ShowProgressRole, false);
+        listItem->setData(DocsetListItemDelegate::ShowProgressRole, false);
     }
 
-    resetProgress();
     delete m_tmpFiles.take(docsetName);
+
+    updateStatus();
 }
 
 void DocsetsDialog::extractionError(const QString &filePath, const QString &errorString)
@@ -481,7 +452,7 @@ void DocsetsDialog::extractionError(const QString &filePath, const QString &erro
 
     QListWidgetItem *listItem = findDocsetListItem(docsetName);
     if (listItem)
-        listItem->setData(ProgressItemDelegate::ShowProgressRole, false);
+        listItem->setData(DocsetListItemDelegate::ShowProgressRole, false);
 
     delete m_tmpFiles.take(docsetName);
 }
@@ -491,7 +462,7 @@ void DocsetsDialog::extractionProgress(const QString &filePath, qint64 extracted
     const QString docsetName = docsetNameForTmpFilePath(filePath);
     QListWidgetItem *listItem = findDocsetListItem(docsetName);
     if (listItem)
-        listItem->setData(ProgressItemDelegate::ValueRole, percent(extracted, total));
+        listItem->setData(DocsetListItemDelegate::ValueRole, percent(extracted, total));
 }
 
 void DocsetsDialog::loadDocsetList()
@@ -519,16 +490,16 @@ void DocsetsDialog::loadDocsetList()
     }
 
     // TODO: Show more user friendly labels, like "5 hours ago"
-    ui->lastUpdatedLabel->setText(fi.lastModified().toString(Qt::SystemLocaleShortDate));
+    const QString updateTime
+            = fi.lastModified().toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat));
+    ui->lastUpdatedLabel->setText(updateTime);
     processDocsetList(jsonDoc.array());
 }
 
 void DocsetsDialog::setupInstalledDocsetsTab()
 {
-    using Registry::ListModel;
-
     ui->installedDocsetList->setItemDelegate(new DocsetListItemDelegate(this));
-    ui->installedDocsetList->setModel(new ListModel(m_application->docsetRegistry(), this));
+    ui->installedDocsetList->setModel(m_docsetRegistry->model());
 
     if (m_isStorageReadOnly) {
         return;
@@ -544,10 +515,11 @@ void DocsetsDialog::setupInstalledDocsetsTab()
 
     QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
     connect(selectionModel, &QItemSelectionModel::selectionChanged,
-            [this, selectionModel]() {
+            this, [this, selectionModel]() {
         ui->removeDocsetsButton->setEnabled(selectionModel->hasSelection());
 
-        for (const QModelIndex &index : selectionModel->selectedRows()) {
+        const auto selectedRows = selectionModel->selectedRows();
+        for (const QModelIndex &index : selectedRows) {
             if (index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
                 ui->updateSelectedDocsetsButton->setEnabled(true);
                 return;
@@ -571,16 +543,16 @@ void DocsetsDialog::setupAvailableDocsetsTab()
 {
     using Registry::DocsetRegistry;
 
-    ui->availableDocsetList->setItemDelegate(new ProgressItemDelegate(this));
+    ui->availableDocsetList->setItemDelegate(new DocsetListItemDelegate(this));
 
-    connect(m_docsetRegistry, &DocsetRegistry::docsetUnloaded, this, [this](const QString name) {
+    connect(m_docsetRegistry, &DocsetRegistry::docsetUnloaded, this, [this](const QString &name) {
         QListWidgetItem *item = findDocsetListItem(name);
         if (!item)
             return;
 
         item->setHidden(false);
     });
-    connect(m_docsetRegistry, &DocsetRegistry::docsetLoaded, this, [this](const QString name) {
+    connect(m_docsetRegistry, &DocsetRegistry::docsetLoaded, this, [this](const QString &name) {
         QListWidgetItem *item = findDocsetListItem(name);
         if (!item)
             return;
@@ -599,23 +571,24 @@ void DocsetsDialog::setupAvailableDocsetsTab()
 
     connect(ui->availableDocsetList, &QListView::activated, this, [this](const QModelIndex &index) {
         // TODO: Cancel download if it's already in progress.
-        if (index.data(ProgressItemDelegate::ShowProgressRole).toBool())
+        if (index.data(DocsetListItemDelegate::ShowProgressRole).toBool())
             return;
 
         ui->availableDocsetList->selectionModel()->select(index, QItemSelectionModel::Deselect);
 
         QAbstractItemModel *model = ui->availableDocsetList->model();
-        model->setData(index, tr("Downloading: %p%"), ProgressItemDelegate::FormatRole);
-        model->setData(index, 0, ProgressItemDelegate::ValueRole);
-        model->setData(index, true, ProgressItemDelegate::ShowProgressRole);
+        model->setData(index, tr("Downloading: %p%"), DocsetListItemDelegate::FormatRole);
+        model->setData(index, 0, DocsetListItemDelegate::ValueRole);
+        model->setData(index, true, DocsetListItemDelegate::ShowProgressRole);
 
         downloadDashDocset(index);
     });
 
     QItemSelectionModel *selectionModel = ui->availableDocsetList->selectionModel();
-    connect(selectionModel, &QItemSelectionModel::selectionChanged, [this, selectionModel]() {
-        for (const QModelIndex &index : selectionModel->selectedRows()) {
-            if (!index.data(ProgressItemDelegate::ShowProgressRole).toBool()) {
+    connect(selectionModel, &QItemSelectionModel::selectionChanged, this, [this, selectionModel]() {
+        const auto selectedRows = selectionModel->selectedRows();
+        for (const QModelIndex &index : selectedRows) {
+            if (!index.data(DocsetListItemDelegate::ShowProgressRole).toBool()) {
                 ui->downloadDocsetsButton->setEnabled(true);
                 return;
             }
@@ -630,18 +603,22 @@ void DocsetsDialog::setupAvailableDocsetsTab()
 
 void DocsetsDialog::enableControls()
 {
-    // Available docsets
-    ui->refreshButton->setEnabled(true);
-
-    if (m_isStorageReadOnly) {
+    if (m_isStorageReadOnly || !m_replies.isEmpty() || !m_tmpFiles.isEmpty()) {
         return;
     }
 
+    // Dialog buttons.
+    ui->buttonBox->setStandardButtons(QDialogButtonBox::Close);
+
+    // Available docsets
+    ui->refreshButton->setEnabled(true);
+
     // Installed docsets
     ui->addFeedButton->setEnabled(true);
     QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
     bool hasSelectedUpdates = false;
-    for (const QModelIndex &index : selectionModel->selectedRows()) {
+    const auto selectedRows = selectionModel->selectedRows();
+    for (const QModelIndex &index : selectedRows) {
         if (index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
             hasSelectedUpdates = true;
             break;
@@ -655,6 +632,12 @@ void DocsetsDialog::enableControls()
 
 void DocsetsDialog::disableControls()
 {
+    // Dialog buttons.
+    if (!m_isStorageReadOnly) {
+        // Always show the close button if storage is read only.
+        ui->buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
+    }
+
     // Installed docsets
     ui->addFeedButton->setEnabled(false);
     ui->updateSelectedDocsetsButton->setEnabled(false);
@@ -680,7 +663,8 @@ QListWidgetItem *DocsetsDialog::findDocsetListItem(const QString &name) const
 
 bool DocsetsDialog::updatesAvailable() const
 {
-    for (Registry::Docset *docset : m_docsetRegistry->docsets()) {
+    const auto docsets = m_docsetRegistry->docsets();
+    for (Registry::Docset *docset : docsets) {
         if (docset->hasUpdate)
             return true;
     }
@@ -696,20 +680,19 @@ QNetworkReply *DocsetsDialog::download(const QUrl &url)
     m_replies.append(reply);
 
     disableControls();
-
-    updateCombinedProgress();
+    updateStatus();
 
     return reply;
 }
 
 void DocsetsDialog::cancelDownloads()
 {
-    for (QNetworkReply *reply : m_replies) {
+    for (QNetworkReply *reply : qAsConst(m_replies)) {
         // Hide progress bar
         QListWidgetItem *listItem
                 = ui->availableDocsetList->item(reply->property(ListItemIndexProperty).toInt());
         if (listItem)
-            listItem->setData(ProgressItemDelegate::ShowProgressRole, false);
+            listItem->setData(DocsetListItemDelegate::ShowProgressRole, false);
 
         if (reply->property(DownloadTypeProperty).toInt() == DownloadDocset)
             delete m_tmpFiles.take(reply->property(DocsetNameProperty).toString());
@@ -717,12 +700,13 @@ void DocsetsDialog::cancelDownloads()
         reply->abort();
     }
 
-    resetProgress();
+    updateStatus();
 }
 
 void DocsetsDialog::loadUserFeedList()
 {
-    for (Registry::Docset *docset : m_docsetRegistry->docsets()) {
+    const auto docsets = m_docsetRegistry->docsets();
+    for (Registry::Docset *docset : docsets) {
         if (!docset->feedUrl().isEmpty()) {
             QNetworkReply *reply = download(QUrl(docset->feedUrl()));
             reply->setProperty(DownloadTypeProperty, DownloadDashFeed);
@@ -745,27 +729,30 @@ void DocsetsDialog::processDocsetList(const QJsonArray &list)
         QJsonObject docsetJson = v.toObject();
 
         Registry::DocsetMetadata metadata(docsetJson);
-        m_availableDocsets.insert(metadata.name(), metadata);
+        m_availableDocsets.insert({metadata.name(), metadata});
     }
 
     // TODO: Move into dedicated method
-    for (const Registry::DocsetMetadata &metadata : m_availableDocsets) {
-        QListWidgetItem *listItem
-                = new QListWidgetItem(metadata.icon(), metadata.title(), ui->availableDocsetList);
+    for (const auto &kv : m_availableDocsets) {
+        const auto &metadata = kv.second;
+
+        auto listItem = new QListWidgetItem(metadata.icon(), metadata.title(), ui->availableDocsetList);
         listItem->setData(Registry::ItemDataRole::DocsetNameRole, metadata.name());
 
-        if (m_docsetRegistry->contains(metadata.name())) {
-            listItem->setHidden(true);
+        if (!m_docsetRegistry->contains(metadata.name())) {
+            continue;
+        }
 
-            Registry::Docset *docset = m_docsetRegistry->docset(metadata.name());
+        listItem->setHidden(true);
 
-            if (metadata.latestVersion() != docset->version()
-                    || metadata.revision() > docset->revision()) {
-                docset->hasUpdate = true;
+        Registry::Docset *docset = m_docsetRegistry->docset(metadata.name());
 
-                if (!m_isStorageReadOnly) {
-                    ui->updateAllDocsetsButton->setEnabled(true);
-                }
+        if (metadata.latestVersion() != docset->version()
+                || metadata.revision() > docset->revision()) {
+            docset->hasUpdate = true;
+
+            if (!m_isStorageReadOnly) {
+                ui->updateAllDocsetsButton->setEnabled(true);
             }
         }
     }
@@ -777,14 +764,14 @@ void DocsetsDialog::downloadDashDocset(const QModelIndex &index)
 {
     const QString name = index.data(Registry::ItemDataRole::DocsetNameRole).toString();
 
-    if (!m_availableDocsets.contains(name) && !m_userFeeds.contains(name))
+    if (m_availableDocsets.count(name) == 0 && !m_userFeeds.contains(name))
         return;
 
     QUrl url;
     if (!m_userFeeds.contains(name)) {
         // No feed present means that this is a Kapeli docset
-        QString urlString = RedirectServerUrl + QString("/d/com.kapeli/%1/latest");
-        url = QUrl(urlString.arg(name));
+        QString urlString = QString(RedirectServerUrl).arg("com.kapeli").arg(name);
+        url = QUrl(urlString);
     } else {
         url = m_userFeeds[name].url();
     }
@@ -817,29 +804,19 @@ void DocsetsDialog::removeDocset(const QString &name)
     }
 }
 
-void DocsetsDialog::updateCombinedProgress()
+void DocsetsDialog::updateStatus()
 {
-    if (m_replies.isEmpty()) {
-        resetProgress();
-        return;
+    QString text;
+
+    if (!m_replies.isEmpty()) {
+        text = tr("Downloading: %n.", nullptr, m_replies.size());
     }
 
-    ui->combinedProgressBar->show();
-    ui->combinedProgressBar->setValue(percent(m_combinedReceived, m_combinedTotal));
-    ui->cancelButton->show();
-}
+    if (!m_tmpFiles.isEmpty()) {
+        text += QLatin1String(" ") + tr("Installing: %n.", nullptr, m_tmpFiles.size());
+    }
 
-void DocsetsDialog::resetProgress()
-{
-    if (!m_replies.isEmpty())
-        return;
-
-    ui->cancelButton->hide();
-    ui->combinedProgressBar->hide();
-    ui->combinedProgressBar->setValue(0);
-
-    m_combinedReceived = 0;
-    m_combinedTotal = 0;
+    ui->statusLabel->setText(text);
 
     enableControls();
 }
@@ -865,5 +842,11 @@ int DocsetsDialog::percent(qint64 fraction, qint64 total)
 
 QString DocsetsDialog::cacheLocation(const QString &fileName)
 {
-    return QDir(Core::FileManager::cacheLocation()).filePath(fileName);
+    return QDir(Core::Application::cacheLocation()).filePath(fileName);
+}
+
+bool DocsetsDialog::isDirWritable(const QString &path)
+{
+    auto file = std::make_unique<QTemporaryFile>(path + QLatin1String("/.zeal_writable_check_XXXXXX.tmp"));
+    return file->open();
 }
