@@ -24,29 +24,32 @@
 #include "settingsdialog.h"
 #include "ui_settingsdialog.h"
 
+#include <qxtglobalshortcut/qxtglobalshortcut.h>
+
 #include <core/application.h>
 #include <core/settings.h>
 
 #include <QDir>
 #include <QFileDialog>
-#include <QWebSettings>
+#include <QWebEngineProfile>
+#include <QWebEngineSettings>
 
 using namespace Zeal;
 using namespace Zeal::WidgetUi;
 
 namespace {
 // QFontDatabase::standardSizes() lacks some sizes, like 13, which QtWK uses by default.
-const int AvailableFontSizes[] = {9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
-                                  20, 22, 24, 26, 28, 30, 32, 34, 36,
-                                  40, 44, 48, 56, 64, 72};
-const QWebSettings::FontFamily BasicFontFamilies[] = {QWebSettings::SerifFont,
-                                                      QWebSettings::SansSerifFont,
-                                                      QWebSettings::FixedFont};
-}
+constexpr int AvailableFontSizes[] = {9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
+                                      20, 22, 24, 26, 28, 30, 32, 34, 36,
+                                      40, 44, 48, 56, 64, 72};
+constexpr QWebEngineSettings::FontFamily BasicFontFamilies[] = {QWebEngineSettings::SerifFont,
+                                                                QWebEngineSettings::SansSerifFont,
+                                                                QWebEngineSettings::FixedFont};
+} // namespace
 
-SettingsDialog::SettingsDialog(QWidget *parent) :
-    QDialog(parent),
-    ui(new Ui::SettingsDialog())
+SettingsDialog::SettingsDialog(QWidget *parent)
+    : QDialog(parent)
+    , ui(new Ui::SettingsDialog())
 {
     ui->setupUi(this);
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
@@ -75,51 +78,54 @@ SettingsDialog::SettingsDialog(QWidget *parent) :
     setTabOrder(ui->defaultFontComboBox, ui->fontSizeComboBox);
     setTabOrder(ui->fontSizeComboBox, ui->serifFontComboBox);
 
-    QWebSettings *webSettings = QWebSettings::globalSettings();
+    // Disable global shortcut settings if not supported.
+    ui->globalHotKeyGroupBox->setEnabled(QxtGlobalShortcut::isSupported());
+
+    QWebEngineSettings *webSettings = QWebEngineProfile::defaultProfile()->settings();
 
     // Avoid casting in each connect.
     auto currentIndexChangedSignal
             = static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged);
 
-    auto syncStandardFont = [this, webSettings](QWebSettings::FontFamily fontFamily,
+    auto syncStandardFont = [this, webSettings](QWebEngineSettings::FontFamily fontFamily,
             const QFont &font) {
         const int index = ui->defaultFontComboBox->currentIndex();
         if (BasicFontFamilies[index] == fontFamily) {
-            webSettings->setFontFamily(QWebSettings::StandardFont, font.family());
+            webSettings->setFontFamily(QWebEngineSettings::StandardFont, font.family());
         }
     };
 
     connect(ui->defaultFontComboBox, currentIndexChangedSignal,
             this, [webSettings](int index) {
         const QString fontFamily = webSettings->fontFamily(BasicFontFamilies[index]);
-        webSettings->setFontFamily(QWebSettings::StandardFont, fontFamily);
+        webSettings->setFontFamily(QWebEngineSettings::StandardFont, fontFamily);
     });
 
     connect(ui->serifFontComboBox, &QFontComboBox::currentFontChanged,
             this, [webSettings, syncStandardFont](const QFont &font) {
-        webSettings->setFontFamily(QWebSettings::SerifFont, font.family());
-        syncStandardFont(QWebSettings::SerifFont, font);
+        webSettings->setFontFamily(QWebEngineSettings::SerifFont, font.family());
+        syncStandardFont(QWebEngineSettings::SerifFont, font);
     });
     connect(ui->sansSerifFontComboBox, &QFontComboBox::currentFontChanged,
             this, [webSettings, syncStandardFont](const QFont &font) {
-        webSettings->setFontFamily(QWebSettings::SansSerifFont, font.family());
-        syncStandardFont(QWebSettings::SansSerifFont, font);
+        webSettings->setFontFamily(QWebEngineSettings::SansSerifFont, font.family());
+        syncStandardFont(QWebEngineSettings::SansSerifFont, font);
     });
     connect(ui->fixedFontComboBox, &QFontComboBox::currentFontChanged,
             this, [webSettings, syncStandardFont](const QFont &font) {
-        webSettings->setFontFamily(QWebSettings::FixedFont, font.family());
-        syncStandardFont(QWebSettings::FixedFont, font);
+        webSettings->setFontFamily(QWebEngineSettings::FixedFont, font.family());
+        syncStandardFont(QWebEngineSettings::FixedFont, font);
     });
 
     connect(ui->fontSizeComboBox, currentIndexChangedSignal, this, [webSettings](int index) {
-        webSettings->setFontSize(QWebSettings::DefaultFontSize, AvailableFontSizes[index]);
+        webSettings->setFontSize(QWebEngineSettings::DefaultFontSize, AvailableFontSizes[index]);
     });
     connect(ui->fixedFontSizeComboBox, currentIndexChangedSignal, this, [webSettings](int index) {
-        webSettings->setFontSize(QWebSettings::DefaultFixedFontSize, AvailableFontSizes[index]);
+        webSettings->setFontSize(QWebEngineSettings::DefaultFixedFontSize, AvailableFontSizes[index]);
     });
     connect(ui->minFontSizeComboBox, currentIndexChangedSignal, this, [webSettings](int index) {
         const int fontSize = index == 0 ? 0 : AvailableFontSizes[index-1];
-        webSettings->setFontSize(QWebSettings::MinimumFontSize, fontSize);
+        webSettings->setFontSize(QWebEngineSettings::MinimumFontSize, fontSize);
     });
 
     loadSettings();
@@ -180,7 +186,7 @@ void SettingsDialog::loadSettings()
     ui->openNewTabAfterActive->setChecked(settings->openNewTabAfterActive);
 
     // Search Tab
-    ui->fuzzySearchCheckBox->setChecked(settings->fuzzySearchEnabled);
+    ui->fuzzySearchCheckBox->setChecked(settings->isFuzzySearchEnabled);
 
     // Content Tab
     for (int i = 0; i < ui->defaultFontComboBox->count(); ++i) {
@@ -197,8 +203,19 @@ void SettingsDialog::loadSettings()
     ui->fixedFontSizeComboBox->setCurrentText(QString::number(settings->defaultFixedFontSize));
     ui->minFontSizeComboBox->setCurrentText(QString::number(settings->minimumFontSize));
 
-    ui->darkModeCheckBox->setChecked(settings->darkModeEnabled);
-    ui->highlightOnNavigateCheckBox->setChecked(settings->highlightOnNavigateEnabled);
+    switch (settings->contentAppearance) {
+    case Core::Settings::ContentAppearance::Automatic:
+        ui->appearanceAutoRadioButton->setChecked(true);
+        break;
+    case Core::Settings::ContentAppearance::Light:
+        ui->appearanceLightRadioButton->setChecked(true);
+        break;
+    case Core::Settings::ContentAppearance::Dark:
+        ui->appearanceDarkRadioButton->setChecked(true);
+        break;
+    }
+
+    ui->highlightOnNavigateCheckBox->setChecked(settings->isHighlightOnNavigateEnabled);
     ui->customCssFileEdit->setText(QDir::toNativeSeparators(settings->customCssFile));
 
     switch (settings->externalLinkPolicy) {
@@ -214,7 +231,6 @@ void SettingsDialog::loadSettings()
     }
 
     ui->useSmoothScrollingCheckBox->setChecked(settings->isSmoothScrollingEnabled);
-    ui->disableAdCheckBox->setChecked(settings->isAdDisabled);
 
     // Network Tab
     switch (settings->proxyType) {
@@ -224,16 +240,23 @@ void SettingsDialog::loadSettings()
     case Core::Settings::ProxyType::System:
         ui->systemProxySettings->setChecked(true);
         break;
-    case Core::Settings::ProxyType::UserDefined:
+    case Core::Settings::ProxyType::Http:
         ui->manualProxySettings->setChecked(true);
+        ui->proxyTypeHttpRadioButton->setChecked(true);
+        break;
+    case Core::Settings::ProxyType::Socks5:
+        ui->manualProxySettings->setChecked(true);
+        ui->proxyTypeSocks5RadioButton->setChecked(true);
         break;
     }
 
-    ui->httpProxy->setText(settings->proxyHost);
-    ui->httpProxyPort->setValue(settings->proxyPort);
-    ui->httpProxyNeedsAuth->setChecked(settings->proxyAuthenticate);
-    ui->httpProxyUser->setText(settings->proxyUserName);
-    ui->httpProxyPass->setText(settings->proxyPassword);
+    ui->proxyHostEdit->setText(settings->proxyHost);
+    ui->proxyPortEdit->setValue(settings->proxyPort);
+    ui->proxyRequiresAuthCheckBox->setChecked(settings->proxyAuthenticate);
+    ui->proxyUsernameEdit->setText(settings->proxyUserName);
+    ui->proxyPasswordEdit->setText(settings->proxyPassword);
+
+    ui->ignoreSslErrorsCheckBox->setChecked(settings->isIgnoreSslErrorsEnabled);
 }
 
 void SettingsDialog::saveSettings()
@@ -256,7 +279,7 @@ void SettingsDialog::saveSettings()
     settings->openNewTabAfterActive = ui->openNewTabAfterActive->isChecked();
 
     // Search Tab
-    settings->fuzzySearchEnabled = ui->fuzzySearchCheckBox->isChecked();
+    settings->isFuzzySearchEnabled = ui->fuzzySearchCheckBox->isChecked();
 
     // Content Tab
     settings->defaultFontFamily = ui->defaultFontComboBox->currentData().toString();
@@ -268,8 +291,15 @@ void SettingsDialog::saveSettings()
     settings->defaultFixedFontSize = ui->fixedFontSizeComboBox->currentData().toInt();
     settings->minimumFontSize = ui->minFontSizeComboBox->currentData().toInt();
 
-    settings->darkModeEnabled = ui->darkModeCheckBox->isChecked();
-    settings->highlightOnNavigateEnabled = ui->highlightOnNavigateCheckBox->isChecked();
+    if (ui->appearanceAutoRadioButton->isChecked()) {
+        settings->contentAppearance = Core::Settings::ContentAppearance::Automatic;
+    } else if (ui->appearanceLightRadioButton->isChecked()) {
+        settings->contentAppearance = Core::Settings::ContentAppearance::Light;
+    } else if (ui->appearanceDarkRadioButton->isChecked()) {
+        settings->contentAppearance = Core::Settings::ContentAppearance::Dark;
+    }
+
+    settings->isHighlightOnNavigateEnabled = ui->highlightOnNavigateCheckBox->isChecked();
     settings->customCssFile = QDir::fromNativeSeparators(ui->customCssFileEdit->text());
 
     if (ui->radioExternalLinkAsk->isChecked()) {
@@ -281,22 +311,28 @@ void SettingsDialog::saveSettings()
     }
 
     settings->isSmoothScrollingEnabled = ui->useSmoothScrollingCheckBox->isChecked();
-    settings->isAdDisabled = ui->disableAdCheckBox->isChecked();
 
     // Network Tab
     // Proxy settings
-    if (ui->noProxySettings->isChecked())
+    if (ui->noProxySettings->isChecked()) {
         settings->proxyType = Core::Settings::ProxyType::None;
-    else if (ui->systemProxySettings->isChecked())
+    } else if (ui->systemProxySettings->isChecked()) {
         settings->proxyType = Core::Settings::ProxyType::System;
-    else if (ui->manualProxySettings->isChecked())
-        settings->proxyType = Core::Settings::ProxyType::UserDefined;
+    } else if (ui->manualProxySettings->isChecked()) {
+        if (ui->proxyTypeSocks5RadioButton->isChecked()) {
+            settings->proxyType = Core::Settings::ProxyType::Socks5;
+        } else {
+            settings->proxyType = Core::Settings::ProxyType::Http;
+        }
+    }
 
-    settings->proxyHost = ui->httpProxy->text();
-    settings->proxyPort = ui->httpProxyPort->text().toUShort();
-    settings->proxyAuthenticate = ui->httpProxyNeedsAuth->isChecked();
-    settings->proxyUserName = ui->httpProxyUser->text();
-    settings->proxyPassword = ui->httpProxyPass->text();
+    settings->proxyHost = ui->proxyHostEdit->text();
+    settings->proxyPort = ui->proxyPortEdit->text().toUShort();
+    settings->proxyAuthenticate = ui->proxyRequiresAuthCheckBox->isChecked();
+    settings->proxyUserName = ui->proxyUsernameEdit->text();
+    settings->proxyPassword = ui->proxyPasswordEdit->text();
+
+    settings->isIgnoreSslErrorsEnabled = ui->ignoreSslErrorsCheckBox->isChecked();
 
     settings->save();
 }
