@@ -29,7 +29,6 @@
 class QCompleter;
 class QEvent;
 class QLabel;
-class QTreeView;
 
 namespace Zeal {
 namespace WidgetUi {
@@ -40,7 +39,6 @@ class SearchEdit : public QLineEdit
 public:
     explicit SearchEdit(QWidget *parent = nullptr);
 
-    void setTreeView(QTreeView *view);
     void clearQuery();
     void selectQuery();
     void setCompletions(const QStringList &completions);
@@ -48,8 +46,6 @@ public:
 protected:
     bool event(QEvent *event) override;
     void focusInEvent(QFocusEvent *event) override;
-    void keyPressEvent(QKeyEvent *event) override;
-    void mousePressEvent(QMouseEvent *event) override;
 
 private slots:
     void showCompletions(const QString &text);
@@ -59,9 +55,7 @@ private:
     int queryStart() const;
 
     QCompleter *m_prefixCompleter = nullptr;
-    QTreeView *m_treeView = nullptr;
     QLabel *m_completionLabel = nullptr;
-    bool m_focusing = false;
 };
 
 } // namespace WidgetUi
